@@ -1,9 +1,7 @@
 package acos;
 ##
-## $Id: acos.pm.in 3507 2016-11-21 17:45:46Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package acos;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,14 +55,12 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 
 # load-time initialization
 sub import {
-    $timeo = 90;
-
     0;
 }
 
@@ -180,6 +170,7 @@ sub ShowAflex {
 	tr/\015//d;
 	next if /^\s*$/;
 	last if (/^$prompt/);
+	return(1) if (/% unrecognized command\./i);	# not in CGN/LSN image
 
 	s/\s*$/\n/;					# trim trailing WS
         ProcessHistory("AFLEX", "", "", "!AFLEX: $_");
