@@ -1,9 +1,7 @@
 package aeos;
 ##
-## $Id: aeos.pm.in 3450 2016-08-25 14:05:44Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package aeos;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -67,7 +59,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 our $found_env;
 our $found_version;
@@ -146,7 +138,7 @@ sub ShowVersion {
 
     while (<$INPUT>) {
 	tr/\015//d;
-	if (/^$prompt/) { $found_version = 1; last};
+	if (/^$prompt/) {$found_version = 1; last};
 	next if (/^(\s*|\s*$cmd\s*)$/);
 	return(1) if /^% Invalid input/;
 	return(0) if ($found_version);		# Only do this routine once
@@ -154,12 +146,12 @@ sub ShowVersion {
 	next if (/uptime is/);			# Older AOS show uptime
         my($key, $val) = split(/:\s+/, $_, 2);
         if (! $val) {
-           # Must be the model name if there's no colon
-           ProcessHistory("COMMENTS", "", "", "!Model: $_" );
-           next;
+	    # Must be the model name if there's no colon
+	    ProcessHistory("COMMENTS", "", "", "!Model: $_" );
+	    next;
         }
         if ($key ne 'Uptime' and $key ne 'Free memory') {
-           ProcessHistory("COMMENTS", "", "", "!$key: $val" );
+	    ProcessHistory("COMMENTS", "", "", "!$key: $val" );
         }
     }
     return(0);
@@ -191,11 +183,11 @@ sub DiffConfig {
 sub ShowEnv {
     my($INPUT, $OUTPUT, $cmd) = @_;
     print STDERR "    In ShowEnv: $_" if ($debug);
-    my($fans, $supplies, $E0) = (0, 0, undef);
+    my($fans, $E0) = (0, undef);
 
     while (<$INPUT>) {
 	tr/\015//d;
-	if (/^$prompt/) { $found_env = 1; last};
+	if (/^$prompt/) {$found_env = 1; last};
 	next if (/^(\s*|\s*$cmd\s*)$/);
 	return(1) if /^% Invalid input/;
 	return(0) if ($found_env);		# Only do this routine once
@@ -206,19 +198,25 @@ sub ShowEnv {
 	}
         /^(System \S+ status is|Action on overheat|Airflow)/ &&
 	    ProcessHistory("COMMENTS","keysort","E0","!$_") && next;
-        if ( /^Fan\s+Status\s+/ ) {
-            $fans = 1;
-            next;
+        if (/^Fan\s+Status\s+/) {
+	    $fans = 1;
+	    next;
         }
         $fans && /^\s*(\d+)\s+(\S+)/ &&
-            ProcessHistory("COMMENTS","keysort","E1","!Fan $1: $2\n") && next;
-        if ( /^Supply\s+Model\s+Capacity/ ) {
-            $fans = 0;
-            $supplies = 1;
-            next;
+	    ProcessHistory("COMMENTS","keysort","E1","!Fan $1: $2\n") && next;
+        if (/^supply\s+model\s+capacity/i) {
+	    while (<$INPUT>) {
+		tr/\015//d;
+		last if (/^\s*/);
+		if (/^$prompt/) {$found_version = 1; last};
+		/^\s*(\d+)\s+(\S+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)/ &&
+		    ProcessHistory("COMMENTS","keysort","E2","!Power Supply $1: $2 $3 $4\n");
+		    next;
+		/^\s*Total\s+\S+\s+(\S+)\s+/ &&
+		    ProcessHistory("COMMENTS","keysort","E2","!Power Total: $1\n") &&
+		    next;;
+	    }
         }
-        $supplies && /^\s*(\d+)\s+(\S+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(.+)/ &&
-            ProcessHistory("COMMENTS","keysort","E2","!Power Supply $1: $2 $3 $4\n" ) && next;
     }
     ProcessHistory("COMMENTS","","","!\n");
     return(0);
@@ -255,8 +253,8 @@ sub ShowBootExt {
 	tr/\015//d;
 	last if (/^$prompt/);
 	next if (/^(\s*|\s*$cmd\s*)$/);
-	return(1) if /^% Invalid input/;
-        return(0) if /^% Error displaying /;
+	return(1) if (/^% Invalid input/);
+        return(0) if (/^% Error displaying /);
 
 	if (!defined($C0)) {
 	    $C0 = 1; ProcessHistory("COMMENTS","","","!\n");
@@ -375,6 +373,9 @@ sub WriteTerm {
 	/^ speed / && $lineauto	&& next; # kill speed on serial lines
 	/^ clockrate /		&& next; # kill clockrate on serial interfaces
 	/^! Time: /		&& next; # EOS gives the time that a "show running" command was run
+
+	next if (/(^\*+$|unsaved changes exist)/i);
+
 	if (/^(enable )?(password|passwd)( level \d+)? / && $filter_pwds >= 1) {
 	    ProcessHistory("ENABLE","","","!$1$2$3 <removed>\n");
 	    next;
