@@ -1,9 +1,7 @@
 package arbor;
 ##
-## $Id: arbor.pm.in 3226 2016-01-15 21:41:54Z heas $
-##
-## rancid 3.6.2
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package arbor;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -62,20 +54,24 @@ use warnings;
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.2;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 #XXX @Exporter::EXPORT = qw($VERSION @commandtable %commands @commands);
 
 # load-time initialization
 sub import {
+    # force a terminal type so as not to confuse the POS with "network".
+    # Seems that it might also support "dumb".
+    $ENV{'TERM'} = "vt100";
+
+    $timeo = 240;			# anlogin timeout in seconds
+
     0;
 }
 
 # post-open(collection file) initialization
 sub init {
-    $timeo = 90;			# anlogin timeout in seconds
-
     # add content lines and separators
     ProcessHistory("","","","#RANCID-CONTENT-TYPE: $devtype\n#\n");
     0;
@@ -95,7 +91,7 @@ TOP: while(<$INPUT>) {
 	if (/^Error:/) {
 	    print STDOUT ("$host anlogin error: $_");
 	    print STDERR ("$host anlogin error: $_") if ($debug);
-	    $clean_run=0;
+	    $clean_run = 0;
 	    last;
 	}
 	while (/[#>]\s*($cmds_regexp)\s*$/) {
@@ -139,6 +135,9 @@ sub ShowConfig {
 	next if (/^(\s*|\s*$cmd\s*)$/);
 	return(1) if (/invalid command/i);
 
+	# some stupid timer error from this half-assed device
+	next if (/comsh: timer error, idle timeout will be unavailable/i);
+
 	# sort mgmt ACLs
 	if (/ip access add (\S+) all (\S+)/) {
 	    ProcessHistory("IPACCESS_$1","$aclsort","$2","$_"); next;
@@ -221,6 +220,12 @@ sub ShowHardware {
 	next if (/^(\s*|\s*$cmd\s*)$/);
 	return(1) if (/invalid command/i);
 
+	# observed intermittently on T4008 running 8.2.0.
+	return(-1) if (/error getting board version/i);
+
+	# some stupid timer error from this half-assed device
+	next if (/comsh: timer error, idle timeout will be unavailable/i);
+
 	next if (/^boot time: /i);
 	next if (/^load averages: /i);
 
@@ -241,6 +246,9 @@ sub ShowVersion {
 	next if (/^(\s*|\s*$cmd\s*)$/);
 	return(1) if (/invalid command/i);
 
+	# some stupid timer error from this half-assed device
+	next if (/comsh: timer error, idle timeout will be unavailable/i);
+
 	ProcessHistory("Version","","","# $_");
     }
     ProcessHistory("Version","","","#\n");
