@@ -1,9 +1,7 @@
 package ciscowlc;
 ##
-## $Id: ciscowlc.pm.in 3444 2016-08-10 16:25:47Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package ciscowlc;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -64,14 +56,13 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 #XXX @Exporter::EXPORT = qw($VERSION @commandtable %commands @commands);
 
 # load-time initialization
 sub import {
-    $timeo = 90;			# wlogin timeout in seconds
     0;
 }
 
@@ -90,10 +81,6 @@ sub inloop {
 
 TOP: while(<$INPUT>) {
 	tr/\015//d;
-	if (/^.*logout(Connection.*closed.)?$/)  {
-	    $clean_run = 1;
-	    last;
-	}
 	if (/^Error:/) {
 	    print STDOUT ("$host wlogin error: $_");
 	    print STDERR ("$host wlogin error: $_") if ($debug);
@@ -114,7 +101,7 @@ TOP: while(<$INPUT>) {
 		last TOP;
 	    }
 	    if (! defined(&{$commands{$cmd}})) {
-		printf(STDERR "$host: undefined function - \"%s\"\n",  
+		printf(STDERR "$host: undefined function - \"%s\"\n",
 		       $commands{$cmd});
 		$clean_run = 0;
 		last TOP;
@@ -126,6 +113,10 @@ TOP: while(<$INPUT>) {
 		last TOP;
 	    }
 	}
+	if (/^.*logout(\s*connection.*closed.*)?$/i) {
+	    $clean_run = 1;
+	    last;
+	}
     }
 }
 
@@ -139,9 +130,11 @@ sub ShowConfig {
     while (<$INPUT>) {
         tr/\015//d;
         tr/\020//d;
+	last if (/^$prompt/);
 
 	next if (/^\s*rogue ap classify/);
-	next if (/^\s*rogue (adhoc|client) alert/);
+	next if (/^\s*rogue (adhoc|client) (alert|unknown)/i);
+	next if (/^\s*interface nat-address management set -?[0-9]{4,}\./);
 
 	$linecnt++;
 
@@ -165,17 +158,14 @@ sub ShowConfig {
 	    ProcessHistory("","","","!$1 <removed>\n"); next;
 	}
 
-	last if (/^$prompt/);
 	next if (/^(\s*|\s*$cmd\s*)$/);
 
 	$linecnt++;
 	if (! /^$prompt/) {
-	    print(STDERR "      ShowConfig Data: $_") if ($debug);
 	    ProcessHistory("","","","$_");
 	}
     }
     ProcessHistory("","","","\n!--WLC End Config Data--!\n");
-    print STDERR "    Exiting ShowConfig: $_" if ($debug);
     # WLC lacks a definitive "end of config" marker.
     if ($linecnt > 5) {
 	$found_end = 1;
@@ -199,20 +189,19 @@ sub ShowSysinfo {
 	if (/^fan status/i) { $skipprocess = 1; }
 	if (/^Number of Active Clients/) { $skipprocess = 1; }
 	if (/^OUI Classification Failure Count/) { $skipprocess = 1; }
-	if (/^(internal|external) temperature/i) { $skipprocess=1; }
+	if (/^(memory|cpu) (current|average)/i) { $skipprocess = 1; }
+	if (/^(mgig|internal|external) temperature/i) { $skipprocess=1; }
 	if (/rpm$/) { $skipprocess = 1; }
 
 	last if (/^$prompt/);
 	next if (/^(\s*|\s*$cmd\s*)$/);
 	if (! /^$prompt/) {
 		if (! $skipprocess) {
-			print STDERR "      ShowSysinfo Data: $_" if ($debug);
 			ProcessHistory("","","","! $_");
 		}
 	}
     }
     ProcessHistory("","","","!\n!WLC Show Sysinfo End\n");
-    print STDERR "    Exiting ShowSysinfo: $_" if ($debug);
     return(0);
 }
 
@@ -227,11 +216,9 @@ sub ShowUdi {
 
 	last if (/^$prompt/);
 	next if (/^(\s*|\s*$cmd\s*)$/);
-	print STDERR "      ShowUdi Data: $_" if ($debug);
 	ProcessHistory("","","","! $_");
     }
     ProcessHistory("","","","!\n!WLC Show Udi End\n");
-    print STDERR "    Exiting ShowSysinfo: $_" if ($debug);
     return(0);
 }
 
