@@ -1,13 +1,11 @@
 package dell;
 ##
-## $Id: dell.pm.in 3226 2016-01-15 21:41:54Z heas $
-##
 ## fnrancid patched to dlrancid to accomplish D-Link support
 ## adapted by: Gavin McCullagh <gmccullagh at gmail.com>
 ##             Griffith College Dublin
 ##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -22,15 +20,9 @@ package dell;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -77,7 +69,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 our $proc;
 our $found_version;
@@ -87,9 +79,6 @@ our $found_version;
 
 # load-time initialization
 sub import {
-    $timeo = 300;		# dllogin timeout in seconds (some of these
-				# devices are remarkably slow to read config)
-
     0;
 }
 
@@ -112,7 +101,7 @@ sub inloop {
 TOP: while(<$INPUT>) {
 	tr/\015//d;
 	# XXX this match is not correct for DELL
-	if (/[>#]\s?exit$/) {
+	if (/[>#]\s?exit(?:$|Connection)/) {
 	    $clean_run = 1;
 	    last;
 	}
