@@ -1,9 +1,7 @@
 package edgemax;
 ##
-## $Id: edgemax.pm.in 3376 2016-04-06 17:52:30Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package edgemax;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,7 +55,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 #XXX @Exporter::EXPORT = qw($VERSION @commandtable %commands @commands);
@@ -147,26 +139,6 @@ sub ShowVersion {
     return(0);
 }
 
-# This routine parses "Dir"
-sub DirSlotN {
-    my($INPUT, $OUTPUT) = @_;
-    my($slot);
-
-    print STDERR "    In DirSlotN: $_" if ($debug);
-
-    while (<$INPUT>) {
-	tr/\015//d;
-	last if (/^$prompt/);
-	next if (/^\s+\^$/);
-        return(1) if (/invalid (input|command) detected/i);
-	next if /^\s*$/;
-
-	ProcessHistory("FLASH","","","!$_");
-    }
-    ProcessHistory("FLASH","","","!\n");
-    return(0);
-}
-
 # This routine parses "show bootvar"
 sub ShowBootvar {
     my($INPUT, $OUTPUT) = @_;
@@ -209,7 +181,7 @@ sub ShowEnvironment {
     my($INPUT, $OUTPUT) = @_;
     print STDERR "    In ShowEnvironment: $_" if ($debug);
 
-    while (<$INPUT>) {
+TOP:while (<$INPUT>) {
 	tr/\015//d;
 	last if (/^$prompt/);
 	next if (/^\s+\^$/);
@@ -229,7 +201,37 @@ sub ShowEnvironment {
 
 	next if (/^fan speed level:/i);
 
-	# trim PS info from:
+	# trim speed/duty from fan info:
+	# Fans:
+	# Unit Fan Description    Type      Speed         Duty level    State
+	# ---- --- -------------- --------- ------------- ------------- --------------
+	# 1    1   Fan-1          Fixed     0             50%           Operational
+	# 1    2   Fan-2          Fixed     0             50%           Operational
+	# OR
+	# Fans:
+	# Unit Fan Description    Type      Speed         Duty
+	# ---- --- -------------- --------- ------------- -------------
+	# 1    1   FAN-1          Fixed     7448
+	/fan duty level:/i && next;
+	if (/^unit\s+fan\s+description\s+type\s+speed\s+duty\s+/i) {
+	    ProcessHistory("ENV","","","!$_");
+	    while (<$INPUT>) {
+		tr/\015//d;
+		last if (/^\s*$/);
+		last TOP if (/^$prompt/);
+
+		s/\s*$/\n/;
+		if (/^(\d+\s+\d+\s+\S+\s+\S+\s+)(\d+\s+\d+%)/) {
+		    my($sdl) = length($2);
+		    my($rem) = $';
+		    $_ = sprintf("%s%".length($2)."s%s", $1, " ", $rem);
+		}
+		ProcessHistory("ENV","","","!$_");
+	    }
+	    next;
+	}
+
+	# trim stats from PS info:
 	# Unit     Power supply   Description        Type          State           Consumed(W)   Voltage(V)  Current(mA)
 	# ----     ------------   ----------------   ----------    -------------- ------------ ------------ ------------
 	# 1        1              PS-1               Fixed         Powering                N/A          N/A          N/A
