@@ -1,9 +1,7 @@
 package exos;
 ##
-## $Id: exos.pm.in 3399 2016-05-24 11:36:20Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package exos;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,7 +55,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 #XXX @Exporter::EXPORT = qw($VERSION @commandtable %commands @commands);
@@ -156,11 +148,14 @@ sub ShowVersion {
 	    ProcessHistory("COMMENTS","keysort","D1","#\n#Bootrom: $1\n")
 	    && next;
 
-#XXX: need to collect this from show vers for ShowSlot where rev #s are excluded
-#SLOT 1 :  702005-06  0025S-00877 CPLD Rev <FF>
-#SLOT 2 :  702005-06  0021S-00131 CPLD Rev 02
-#SLOT 3 :  702009-06  0024S-00170 CPLD Rev <FF>
-#SLOT 4 :  702009-06  0024S-00319 CPLD Rev <FF>
+	# collect slot info rev #s, etc.
+	# SLOT 1 :  702005-06  0025S-00877 CPLD Rev <FF>
+	# SLOT 2 :  702005-06  0021S-00131 CPLD Rev 02
+	# SLOT 3 :  702009-06  0024S-00170 CPLD Rev <FF>
+	# SLOT 4 :  702009-06  0024S-00319 CPLD Rev <FF>
+	/^(?:Slot-\d|Switch)\s+:\s+[^\s]+/i &&
+	    ProcessHistory("COMMENTS","keysort","D2","#$_") &&
+	    next;
     }
     return(0);
 }
@@ -208,6 +203,21 @@ sub ShowDiag {
     return(0);
 }
 
+# This routine parses "show policy detail"
+sub ShowPolicy {
+    my($INPUT, $OUTPUT, $cmd) = @_;
+    print STDERR "    In ShowPolicy: $_" if ($debug);
+
+    while (<$INPUT>) {
+	tr/\015//d;
+	last if (/^$prompt/);
+	next if (/^(\s*|\s*$cmd\s*)$/);
+
+	ProcessHistory("POLICY","","","# $_");
+    }
+    return(0);
+}
+
 # This routine parses "show slot"
 sub ShowSlot {
     my($INPUT, $OUTPUT, $cmd) = @_;
