@@ -1,9 +1,7 @@
 package foundry;
 ##
-## $Id: foundry.pm.in 3424 2016-07-12 14:43:44Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package foundry;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,7 +55,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 #XXX @Exporter::EXPORT = qw($VERSION @commandtable %commands @commands);
@@ -310,11 +302,13 @@ sub WriteTerm {
 	# filter out any RCS/CVS tags to avoid confusing local CVS storage
 	s/\$(Revision|Id):/ $1:/;
 	# sort secure-mac-addresses.  Note: There is no way to determine which
-	# have been added dynamically vs statically.  Thus, sort them?
-	/secure-mac-address (\S+)/ &&
-	    ProcessHistory("SECMAC","keysort","$1","$_") && next;
-	# OR filter them? XXX
-	#next if (/secure-mac-address/);
+	# have been added dynamically vs statically.  Filter if FILTER_OSC > YES
+	if (/secure-mac-address (\S+)/) {
+	    if ($filter_osc < 2) {
+		ProcessHistory("SECMAC","keysort","$1","$_");
+	    }
+ 	    next;
+	}
 
 	# order access-lists
 	/^access-list\s+(\d+)\s+(perm|deny)\s+(\d\S+)(\/\d+)\s*$/ &&
