@@ -1,9 +1,7 @@
 package iosxr;
 ##
-## $Id: iosxr.pm.in 3463 2016-09-12 23:04:42Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package iosxr;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,7 +55,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 our $proc;
 our $found_env;
@@ -292,10 +284,19 @@ sub ShowVersion {
 			       "!CPU:$slave $cpu$_$slaveslot\n");
 	    }
 	    next;
+	} elsif (/^cisco (\S+(?:\sseries)?)\s+(?:\(([^)]*)\)\s+processor)/i) {
+	    $proc = $1;
+	    my($cpu) = $2;
+	    my($device) = "router";
+
+	    print STDERR "TYPE = $type\n" if ($debug);
+	    ProcessHistory("COMMENTS","keysort","A1",
+		"!Chassis type: $proc - a $proc $device\n");
+	    if (defined($cpu) && length($cpu)) {
+		ProcessHistory("COMMENTS","keysort","A3",
+			       "!CPU:$cpu\n");
+	    }
 	}
-	/^(\d+[kK]) bytes of multibus/ &&
-	    ProcessHistory("COMMENTS","keysort","B2",
-		"!Memory: multibus $1\n") && next;
 	/^(\d+[kK]) bytes of (non-volatile|NVRAM)/ &&
 	    ProcessHistory("COMMENTS","keysort","B3",
 		"!Memory: nvram $1\n") && next;
@@ -338,10 +339,9 @@ sub ShowVersion {
 }
 
 # This routine parses "admin show diag".
-# This will create arrays for hw info.
 sub AdminShowDiag {
     my($INPUT, $OUTPUT, $cmd) = @_;
-    print STDERR "    In ShowDiag: $_" if ($debug);
+    print STDERR "    In AdminShowDiag: $_" if ($debug);
 
     while (<$INPUT>) {
 	tr/\015//d;
@@ -412,10 +412,10 @@ sub AdminShowRunning {
     return(0);
 }
 
-# This routine parses "admin show hw-module fpd location all".
-sub ShowHWfpd {
+# This routine parses "show diag".
+sub ShowDiag {
     my($INPUT, $OUTPUT, $cmd) = @_;
-    print STDERR "    In ShowRunning: $_" if ($debug);
+    print STDERR "    In ShowDiag: $_" if ($debug);
 
     while (<$INPUT>) {
 	tr/\015//d;
@@ -435,19 +435,22 @@ sub ShowHWfpd {
 	next if (/^\S+ connected from 127.0.0.1 using console /);
 	next if (/# terminal length 0$/);
 
-	/^notes:/i && last;
+	s/^NODE //;
+	# wtf are these?
+	next if (/(New Deviation|UDI_VID|Board State)/);
 
-	ProcessHistory("FPD","","","!$_");
+	ProcessHistory("SLOT","","","!$_");
     }
-    ProcessHistory("FPD","","","!\n");
+    ProcessHistory("SLOT","","","!\n");
     return(0);
 }
 
-# This routine parses "admin show redundancy"
-sub ShowRedundancy {
+
+# This routine parses "admin show hw-module fpd location all" and
+# "show hw-module fpd" (eXR/eXR-lite).
+sub ShowHWfpd {
     my($INPUT, $OUTPUT, $cmd) = @_;
-    my($slave, $slaveslot);
-    print STDERR "    In ShowRedundancy: $_" if ($debug);
+    print STDERR "    In ShowRunning: $_" if ($debug);
 
     while (<$INPUT>) {
 	tr/\015//d;
@@ -459,20 +462,54 @@ sub ShowRedundancy {
 	return(1) if (/(Invalid (input|command) detected|Type help or )/i);
 	next if (/^(\e\[.\d+h)?sysadmin-vm:[^#]+# /i);			# NCS
 	return(1) if (/^(-+\^|syntax error: unknown argument)$/i);	# NCS
+	return(-1) if (/command authorization failed/i);
 
-	if (/^Version information for secondary in slot (\d+):/) {
-	    $slave = " Slave:";
-	    $slaveslot = ", slot $1";
-	    next;
-	}
+	# skip blank lines
+	/^$/ && next;
 
-	/^IOS .* Software \(([A-Za-z0-9_-]*)\), .*Version\s+(.*)$/ &&
-	    ProcessHistory("COMMENTS","keysort","F1",
-		"!Image:$slave Software: $1, $2\n") && next;
-	/^Compiled (.*)$/ &&
-	    ProcessHistory("COMMENTS","keysort","F3",
-		"!Image:$slave Compiled: $1\n") && next;
+	# NCS junk
+	next if (/^\S+ connected from 127.0.0.1 using console /);
+	next if (/# terminal length 0$/);
+
+	/^notes:/i && last;
+
+	# remove trailing WS
+	s/\s+$/\n/;
+
+	ProcessHistory("FPD","","","!$_");
     }
+    ProcessHistory("FPD","","","!\n");
+    return(0);
+}
+
+# This routine parses "show redundancy"
+sub ShowRedundancy {
+    my($INPUT, $OUTPUT, $cmd) = @_;
+    my($slave, $slaveslot);
+    print STDERR "    In ShowRedundancy: $_" if ($debug);
+
+TOP: while (<$INPUT>) {
+	tr/\015//d;
+	last if (/^$prompt/);
+	next if (/^(\s*|\s*$cmd\s*)$/);
+	# filter the bloody cmd-line timestamp
+	next if (/^\w{3} \w{3,4} {1,3}\d{1,2} {1,2}\d{1,2}:\d+:\d+\.\d+ \S+$/);
+	return(1) if (/Line has invalid autocommand /);
+	return(1) if (/(Invalid (input|command) detected|Type help or )/i);
+	next if (/^(\e\[.\d+h)?sysadmin-vm:[^#]+# /i);			# NCS
+	return(1) if (/^(-+\^|syntax error: unknown argument)$/i);	# NCS
+
+	# filter reload/boot info
+	if (/^reload and boot info/i) {
+	    while (<$INPUT>) {
+		tr/\015//d;
+		last TOP if (/^$prompt/);
+		last if (/^\s*$/);
+	    }
+	}
+	ProcessHistory("COMMENTS","keysort","F3", "!$_");
+    }
+    ProcessHistory("COMMENTS","keysort","F3", "!\n");
     return(0);
 }
 
@@ -492,6 +529,8 @@ sub ShowInstallActive {
 	return(1) if (/(Invalid (input|command) detected|Type help or )/i);
 	return(1) if (/^(-+\^|syntax error: unknown argument)$/i);	# NCS
 	return(-1) if (/command authorization failed/i);
+	return(-1) if (/install director is not up or ready/i);# temp error
+	return(-1) if (/failed to output install information /i);# temp error
 
 	# FPD upgrade in progress on some hardware, reload/configuration change
 	# on those is not recommended as it might cause HW programming failure
@@ -507,6 +546,38 @@ sub ShowInstallActive {
     return(0);
 }
 
+# This routine parses "show memory summary"
+sub ShowMemorySum {
+    my($INPUT, $OUTPUT, $cmd) = @_;
+    print STDERR "    In ShowMemorySum: $_" if ($debug);
+
+    # RP/0/RSP0/CPU0:r00.lab#show memory summary
+    #
+    # node:      node0_RSP0_CPU0
+    # ------------------------------------------------------------------
+    # 
+    #  Physical Memory: 27426M total (22146M available)
+    #  Application Memory : 27426M (21819M available)
+    #  Image: 4M (bootram: 0M)
+    #  Reserved: 0M, IOMem: 0M, flashfsys: 0M
+    #  Total shared window: 140M
+    while (<$INPUT>) {
+	tr/\015//d;
+	last if (/^$prompt/);
+	next if (/^(\s*|\s*$cmd\s*)$/);
+	# filter the bloody cmd-line timestamp
+	next if (/^\w{3} \w{3,4} {1,3}\d{1,2} {1,2}\d{1,2}:\d+:\d+\.\d+ \S+$/);
+	return(1) if (/Line has invalid autocommand /);
+	return(1) if (/(Invalid (input|command) detected|Type help or )/i);
+	return(-1) if (/command authorization failed/i);
+
+	/physical memory:\s+(\d+[GMKB])\s+total/i &&
+	    ProcessHistory("COMMENTS","keysort","B1",
+			   "!Memory: main $1\n") && next;
+    }
+    return(0);
+}
+
 # This routine parses "admin show env all"
 sub ShowEnv {
     my($INPUT, $OUTPUT, $cmd) = @_;
@@ -577,6 +648,7 @@ sub DirSlotN {
 	return(1) if (/^(-+\^|syntax error: unknown argument)$/i);	# NCS
 	return(1) if (/(No such device|Error Sending Request)/i);
 	return(1) if (/: path doesnot exists/i);	# NCS
+	return(1) if (/: Path does not exist/i);	# NCS
 	return(1) if (/\%Error: No such file or directory/);
 	return(1) if (/\%Error: No route to host/);
 	return(1) if (/No space information available/);
@@ -588,29 +660,7 @@ sub DirSlotN {
 	return(-1) if (/\%Error opening \S+:\S+ \(Device or resource busy\)/i);
 	return(-1) if (/command authorization failed/i);
 	return(1) if (/(Open device \S+ failed|Error opening \S+:)/);
-	# filter frequently changing files from IOX bootflash, hardiska,
-	# and nvram
-	if ($dev =~ /(bootflash|disk0|harddisk|nvram)/) {
-	    if (/\s(aaa|ce_switch.log|errmsg_cont|genstr_cont|temp_cont|temp_static_data|uptime_cont|volt_cont)\s*$/) {
-		# change
-		# 57          -rw-  23100       <no date>              volt_cont
-		# 614788      drwx  4096        Fri Aug 20 12:06:25 2010  temp_cont
-		# to
-		# 57          -rw-              <no date>              volt_cont
-		# 614788      drwx                                        temp_cont
-		if (/(\s*\d+\s+\S+\s+)(\d+)(\s+)(<no date> )/) {
-		    my($a, $sz, $c, $dt, $rem) = ($1, $2, $3, $4, $');
-		    my($szl) = length($sz);
-		    my($fmt) = "%s%-". $szl ."s%s%s%s";
-		    $_ = sprintf($fmt, $a, "", $c, $dt, $rem);
-		} elsif (/(\s*\d+\s+\S+\s+)(\d+)(\s+)(\w+ \w+\s+\d+ \d+:\d+:\d+ \d+)/) {
-		    my($a, $sz, $c, $dt, $rem) = ($1, $2, $3, $4, $');
-		    my($szl, $dtl) = (length($sz), length($dt));
-		    my($fmt) = "%s%-". $szl ."s%s%-". $dtl ."s%s";
-		    $_ = sprintf($fmt, $a, "", $c, "", $rem);
-		}
-	    }
-	}
+
 	# Filter dhcp database
 	if (/dhcp_[^. ]*\.txt/) {
 	    next;
@@ -619,23 +669,73 @@ sub DirSlotN {
 	if ($proc =~ /ASR9K/ && /dlbg\.txt/) {
 	    next;
 	}
-	if (/.*\((\d+) (k?bytes) free\)/) {
- 	    my($tmp) = $1;
- 	    my($sz) = $2;
-	    $tmp *= 1024 if ($sz eq "kbytes");	# NCS summarizes in kbytes
-	
-	    if ($tmp >= (1024 * 1024 * 1024)) {
-		$tmp = int($tmp / (1024 * 1024 * 1024));
-		s/$1 $sz free/$tmp GB free/;
-	    } elsif ($tmp >= (1024 * 1024)) {
-		$tmp = int($tmp / (1024 * 1024));
-		s/$1 $sz free/$tmp MB free/;
+	# filter frequently changing files from IOX bootflash, hardiska,
+	# and nvram
+	if ($dev =~ /(bootflash|disk[0-9]|harddisk|nvram)/) {
+	    if (/\s(\.python-history|aaa|\.bash_history|.sppdc)\s*$/ ||
+	        /\s(ce_switch.log\S*|cisco_support|errmsg_cont)\s*$/ ||
+	        /\s(genstr_cont|temp_cont|temp_cont|temp_static_data)\s*$/ ||
+	        /\s(uptime_cont|volt_cont|volt_hist)\s*$/) {
+		# change
+		# 57          -rw-  23100       <no date>              volt_cont
+		# 614788      drwx  4096        Fri Aug 20 12:06:25 2010  temp_cont
+		# to
+		# -rw-              <no date>              volt_cont
+		# drwx                                        temp_cont
+		if (/\s*\d+\s+(\S+\s+)(\d+)(\s+)(<no date>)(\s+)/) {
+		    my($a, $sz, $c, $dt, $d, $rem) = ($1, $2, $3, $4, $5, $');
+		    my($szl) = length($sz);
+		    my($fmt) = "%s%-". $szl ."s%s%s%s%s";
+		    $_ = sprintf($fmt, $c, $dt, $d, $rem);
+		    ProcessHistory("FLASH","keysort",$rem,"!Flash: $dev: $_");
+		    next;
+		} elsif (/\s*\d+\s+(\S+\s+\d+\s+)(\d+\s+\w+\s+\d+\s+\d+:\d+)/ ||
+		         /\s*\d+\s+(\S+\s+\d+\s+)(\d+\s+\w+\s+\d+\s+\d{4})/) {
+		    # XR >= 6.3; dir disk0:, but harddisk: is diff format.  wtf
+		    # drop fileno size, & date.
+		    # " 8002 drwxr-xr-x 2 4096 Jan 17 15:27 np"
+		    my($perm, $dt, $rem) = ($1, $2, $');
+		    my($dtl) = length($dt);
+		    my($fmt) = "%s%-". $dtl ."s%s";
+		    $_ = sprintf($fmt, $perm, "", $rem);
+		    ProcessHistory("FLASH","keysort",$rem,"!Flash: $dev: $_");
+		    next;
+		} elsif (/\s*\d+\s+(\S+\s+)(\d+)(\s+)(\w+ \w+\s+\d+ \d+:\d+:\d+ \d+)/) {
+		    my($b, $sz, $c, $dt, $rem) = ($1, $2, $3, $4, $');
+		    my($szl, $dtl) = (length($sz), length($dt));
+		    my($fmt) = "%s%-". $szl ."s%s%-". $dtl ."s%s";
+		    $_ = sprintf($fmt, $b, "", $c, "", $rem);
+		    ProcessHistory("FLASH","keysort",$rem,"!Flash: $dev: $_");
+		    next;
+		}
 	    } else {
-		$tmp = int($tmp / 1024);
-		s/$1 $sz free/$tmp KB free/;
+		if (/\s*\d+\s+(\S+\s+)(\d+)(\s+)(<no date>\s+)/) {
+		    my($sz, $c, $dt, $d, $rem) = ($1, $2, $3, $4, $');
+		    ProcessHistory("FLASH","keysort",$rem,"!Flash: $dev: $sz$c$dt$d$rem");
+		    next;
+		} elsif (/\s*\d+\s+(\S+\s+\d+\s+)(\d+\s+\w+\s+\d+\s+\d+:\d+)/ ||
+		         /\s*\d+\s+(\S+\s+\d+\s+)(\d+\s+\w+\s+\d+\s+\d{4})/) {
+		    # XR >= 6.3; dir disk0:, but harddisk: is diff format.  wtf
+		    my($perm, $dt, $rem) = ($1, $2, $');
+		    ProcessHistory("FLASH","keysort",$rem,"!Flash: $dev: $perm$dt$rem");
+		    next;
+		} elsif (/\s*\d+\s+(\S+\s+)(\d+)(\s+)(\w+ \w+\s+\d+ \d+:\d+:\d+ \d+)/) {
+		    # XR < 6.3 & etc.
+		    my($b, $sz, $c, $dt, $rem) = ($1, $2, $3, $4, $');
+		    ProcessHistory("FLASH","keysort",$rem,"!Flash: $dev: $b$sz$c$dt$rem");
+		    next;
+		}
 	    }
 	}
 
+	# XR: 822083584 bytes total (821081600 bytes free)
+	# eXR: 990484 kbytes total (813208 kbytes free)
+	if (/^\s*(\d+ k?bytes) total\s+\((\d+ k?bytes) free\)/i) {
+	    ProcessHistory("FLASH","","","!Flash: $dev: " .
+			   diskszsummary($1, $2) . "\n");
+	    next;
+	}
+
 	ProcessHistory("FLASH","","","!Flash: $dev: $_");
     }
     ProcessHistory("","","","!\n");
@@ -767,7 +867,7 @@ sub ShowDebug {
     return(0);
 }
 
-# This routine parses "admin show install summary"
+# This routine parses "admin show install active"
 sub ShowInstallSummary {
     my($INPUT, $OUTPUT, $cmd) = @_;
     print STDERR "    In ShowInstallSummary: $_" if ($debug);
@@ -789,6 +889,7 @@ sub ShowInstallSummary {
 	return(-1) if (/command authorization failed/i);
 	return(-1) if (/inventory summary cannot be output/i);	# temp error
 	return(-1) if (/currently affected by install operation/i);# temp error
+	return(-1) if (/install director is not up or ready/i);# temp error
 
 	# skip blank lines
 	next if (/^\s*$/);
@@ -796,7 +897,7 @@ sub ShowInstallSummary {
 	next if (/^\S+ connected from 127.0.0.1 using console /);
 	next if (/# terminal length 0$/);
 
-	ProcessHistory("COMMENTS", "keysort", "C15", "! $_");
+	ProcessHistory("COMMENTS", "keysort", "C15", "!Image: $_");
     }
     ProcessHistory("COMMENTS", "keysort", "C10", "!\n");
     ProcessHistory("COMMENTS", "keysort", "C19", "!\n");
@@ -833,8 +934,16 @@ sub ShowInventory {
 	    ProcessHistory("INVENTORY","","", sprintf("!%-30s %s\n", $1, $2));
 	    next;
 	}
+
+	# XR flavor:
+	# NAME: "module 0/RSP0/CPU0", DESCR: "ASR9K Route Switch Processor with 880G/slot Fabric and 32GB"
+	# PID: A9K-RSP880-SE, VID: V01, SN: FOC1911VC2S
+	#
+	# eXR flavor:
+	# NAME: "0/RSP0-Motherboard", DESCR: "Motherboard Module"
+	# PID: N/A               , VID: N/A , SN: N/A
 	# split PID/VID/SN line
-	if (/^PID: (\S*)\s*, VID: (\S*)\s*, SN: (\S*)\s*$/) {
+	if (/^ ?PID: (\S*)\s*, VID: (\S*)\s*,\s* SN: (\S*)\s*$/) {
 	    my($pid,$vid,$sn) = ($1, $2, $3);
 	    my($entries) = "";
 	    # filter <empty>, "0x" and "N/A" lines
@@ -892,6 +1001,32 @@ sub ShowLicense {
 	next if (/^\S+ connected from 127.0.0.1 using console /);
 	next if (/# terminal length 0$/);
 
+	# filter the BS from license broker
+	next if (/(renewal attempt|communication attempt):/i);
+	next if (/next registration attempt:/i);		# timestamp
+	next if (/(period used:|requested time:)/i);		# show lic feature
+	if (/(^\s*(evaluation )?period (left|remaining):\s*)\d+/i) {
+	    ProcessHistory("COMMENTS","keysort","LICENSE","! $1<limited>\n");
+	    next;
+	}
+
+	# drop license counts
+	next if (/license usage:/i);
+	if (/(.*)count status/i) {
+	    my($hdr) = $1;
+	    my($len) = length($hdr);
+
+	    $hdr =~ s/\s*$//;
+	    ProcessHistory("COMMENTS", "keysort", "C20", "! $hdr\n");
+	    while (<$INPUT>) {
+		tr/\015//d;
+		return(0) if (/^$prompt/);
+
+		s/^(.{1,$len}).*/$1/;
+		ProcessHistory("COMMENTS", "keysort", "C20", "! $_");
+	    }
+	    next;
+	}
 	ProcessHistory("COMMENTS", "keysort", "C20", "! $_");
     }
     ProcessHistory("COMMENTS", "keysort", "C20", "!\n");
