@@ -1,9 +1,7 @@
 package mrv;
 ##
-## $Id: mrv.pm.in 3406 2016-06-14 22:23:28Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package mrv;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,7 +55,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 our $found_version;
 
@@ -72,8 +64,6 @@ our $found_version;
 
 # load-time initialization
 sub import {
-    $timeo = 90;                       # clogin timeout in seconds
-
     # force a terminal type so as not to confuse the POS
     $ENV{'TERM'} = "vt100";
 
@@ -177,8 +167,8 @@ sub ShowVersion {
 # This will create arrays for hw info.
 sub ShowChassis {
     my($INPUT, $OUTPUT, $cmd) = @_;
-    my($newfmt) = 0;
     # Skip if this is not a 1200n.
+    my($newfmt) = 0;
     print STDERR "    In ShowChassis: $_" if ($debug);
 
     while (<$INPUT>) {
@@ -210,6 +200,28 @@ sub ShowChassis {
     return(0);
 }
 
+# This routine parses "show inventory" for the gsr
+# This will create arrays for hw info.
+sub ShowInventory {
+    my($INPUT, $OUTPUT, $cmd) = @_;
+    print STDERR "    In ShowInventory: $_" if ($debug);
+
+    while (<$INPUT>) {
+	tr/\015//d;
+	last if (/^$prompt/);
+	next if (/^(\s*$cmd\s*)$/);
+	return(1) if /(Invalid input detected|Type help or )/;
+	return(-1) if (/command authorization failed/i);
+
+	s/\s+$//;
+
+	ProcessHistory("COMMENTS","keysort","C1","!$_\n");
+    }
+    ProcessHistory("COMMENTS","","","!\n");
+    return(0);
+}
+
+
 # This routine parses "show slots".
 sub ShowSlots {
     my($INPUT, $OUTPUT, $cmd) = @_;
