@@ -1,9 +1,7 @@
 package nxos;
 ##
-## $Id: nxos.pm.in 3475 2016-10-25 21:18:22Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package nxos;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -63,15 +55,9 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 our $proc;
-our $ios;
-our $found_version;
-our $found_env;
-our $found_diag;
-our $config_register;			# configuration register value
-
 our $type;				# device model, from ShowVersion
 
 our $C0;				# output formatting control
@@ -90,11 +76,6 @@ sub import {
 # post-open(collection file) initialization
 sub init {
     $proc = "";
-    $found_version = 0;
-    $found_env = 0;
-    $found_diag = 0;
-    $config_register = undef;		# configuration register value
-
     $type = undef;			# device model, from ShowVersion
 
     $C0 = 0;				# output formatting control
@@ -182,7 +163,7 @@ sub ShowVersion {
 	return(-1) if (/\% Permission denied/);
 	return(-1) if (/command authorization failed/i);
 
-	if (/^Cisco Nexus Operating System/) { $type = "NXOS";}
+	if (/^Cisco (Nexus|Storage Area Networking) Operating System/) { $type = "NXOS";}
 	if (/^Software$/) {
 	    while (<$INPUT>) {
 		tr/\015//d;
@@ -313,6 +294,7 @@ sub ShowRedundancy {
 	return(1) if /Line has invalid autocommand /;
 	return(1) if /(Invalid input detected|Type help or )/;
 	return(1) if (/\% Invalid command at /);
+	return(1) if (/.* command is not support/i);
 	return(-1) if (/No token match at /);	# 1000v
 	return(-1) if (/\% Permission denied/);
 	return(-1) if (/command authorization failed/i);
@@ -337,6 +319,7 @@ sub ShowEnv {
 	return(1) if /Line has invalid autocommand /;
 	return(1) if /(Invalid input detected|Type help or )/;
 	return(1) if (/\% Invalid command at /);
+	return(1) if (/ .* command is not support/i);
 	return(1) if (/No token match at /);	# 1000v
 	return(-1) if (/\% Permission denied/);
 	return(-1) if (/command authorization failed/i);
@@ -346,6 +329,8 @@ sub ShowEnv {
 	if (/(control temperature|monitor temperature)/i) {
 	    ProcessHistory("COMMENTS","","","!Env: $_");
 	    while (<$INPUT>) {
+		tr/\015//d;
+		goto EndShowEnv if (/^$prompt/);
 		if (/(.*\s+\d+\s+\d+\s+)(\d+)(.*$)/) {
 		    $_ = sprintf("%s%-". length($2)."s%s\n", $1, "", $3);
 		}
@@ -356,6 +341,7 @@ sub ShowEnv {
 	}
 	ProcessHistory("COMMENTS","","","!Env: $_");
     }
+EndShowEnv:
     ProcessHistory("COMMENTS","","","!\n");
     return(0);
 }
@@ -376,7 +362,7 @@ sub ShowEnvTemp {
 	return(-1) if (/\% Permission denied/);
 	return(-1) if (/command authorization failed/i);
 
-	s/ +$//;	# Drop trailing ' '
+	s/ +$//;	# Drop trailing spaces
 
 # Cut out CurTemp - drop the 2nd to last field.
 #--------------------------------------------------------------------
@@ -387,6 +373,8 @@ sub ShowEnvTemp {
 	if (/(control temperature|monitor temperature)/i) {
 	    ProcessHistory("COMMENTS","","","!Env: $_");
 	    while (<$INPUT>) {
+		tr/\015//d;
+		goto EndShowEnvTemp if (/^$prompt/);
 		if (/(.*\s+\d+\s+\d+\s+)(\d+)(.*$)/) {
 		    $_ = sprintf("%s%-". length($2)."s%s\n", $1, "", $3);
 		}
@@ -402,6 +390,7 @@ sub ShowEnvTemp {
 	    ProcessHistory("COMMENTS","","","!Env: $_");
 	}
     }
+EndShowEnvTemp:
     ProcessHistory("COMMENTS","","","!\n");
     return(0);
 }
@@ -464,12 +453,12 @@ sub ShowEnvPower {
 # Total Power Input (actual draw)                             161.00 W
 # Total Power Allocated (budget)                                N/A
 # Total Power Available for additional modules                  N/A  
-# 
+#
 # nexus#
 	if (/(.* +)(\d+ W +\d+ W)( +\d+ W.+)/) {
 	    $_ = sprintf("%s%-". length($2)."s%s\n", $1, "", $3);
-	} elsif (/(.* +)(\d+ W)( +\d+ W.+)/) {
-	    $_ = sprintf("%s%-". length($2)."s%s\n", $1, "", $3);
+	} elsif (/(.* +)(\d+(\.\d+)? W)( +\d+(\.\d+)? W.+)/) {
+	    $_ = sprintf("%s%-". length($2)."s%s\n", $1, "", $4);
 	}
 
 	/actual draw/ && next;	# Drop changing total power output.
@@ -510,6 +499,7 @@ sub ShowBoot {
 sub DirSlotN {
     my($INPUT, $OUTPUT, $cmd) = @_;
     print STDERR "    In DirSlotN: $_" if ($debug);
+    my($free);
 
     my($dev) = (/\s([^\s]+):/);
 
@@ -534,11 +524,21 @@ sub DirSlotN {
 	# Drop ee.log
 	/\s+ee\.log(?:\..*)?$/ && next;
 
-	# Drop accounting.log
-	/\s+accounting\.log$/ && next;
+	# Drop bootvar_debug log files
+	/\s+bootvar_debug\./ && next;
+
+	# Drop accounting.log*
+	/\s+accounting.log.*/ && next;
+
+	# Drop vtp_debug.log and vtp_debug_old.log CDETS bug CSCuy87611
+	/\s+vtp_debug(_old)?\.log$/ && next;
+
+	# Drop bcm_mem_lock_trace.log
+	/\s+bcm_mem_lock_trace\.log$/ && next;
 
 	next if (/BufferMonitor-1HourData/);
-	if (/ log\/$/) {
+
+	if (/( debug_logs| log)\/$/) {
 	    # change
 	    #         8192    Jan 08 14:05:05 2015  log/
 	    # to
@@ -551,9 +551,13 @@ sub DirSlotN {
 	    }
 	}
 
-	if (/^\s*(\d+) bytes /) {
-	    my($tmp) = int($1 / (1024 * 1024));
-	    s/$1 bytes /$tmp MB /;
+	if (/^\s*(\d+) bytes free/i) {
+	    $free = $1;
+	    next;
+	} elsif (/^\s*(\d+) bytes used/i) {
+	    next;
+	} elsif (/^\s*(\d+) bytes total/i) {
+	    $_ = diskszsummary($1, $free, undef) . "\n";
 	}
 
 	ProcessHistory("COMMENTS","","","!Flash: $dev: $_");
@@ -588,6 +592,33 @@ sub ShowModule {
     return(0);
 }
 
+# This routine parses "show interface transceiver".
+sub ShowIntTransceiver {
+    my($INPUT, $OUTPUT, $cmd) = @_;
+    print STDERR "    In ShowtTransceiver: $_" if ($debug);
+
+    while (<$INPUT>) {
+	tr/\015//d;
+	return if (/^\s*\^$/);
+	last if (/^$prompt/);
+	next if (/^(\s*|\s*$cmd\s*)$/);
+	return(1) if /Line has invalid autocommand /;
+	return(1) if /(Invalid input detected|Type help or )/;
+	return(1) if (/\% Invalid command at /);
+	return(-1) if (/No token match at /);	# 1000v
+	return(-1) if (/\% Permission denied/);
+	return(-1) if (/command authorization failed/i);
+
+	# filter out oscillating data from transceivers
+	next if (/(Temperature|Current|Power|Voltage)\s+:/);
+
+	ProcessHistory("COMMENTS","","","!$_");
+    }
+    ProcessHistory("COMMENTS","","","!\n");
+
+    return(0);
+}
+
 # This routine parses "show inventory".
 sub ShowInventory {
     my($INPUT, $OUTPUT, $cmd) = @_;
@@ -660,7 +691,7 @@ sub ShowVTP {
 	    s/^$1\s{$len}//;
 	}
 
-	# Nexus 5k and 1000v do note support vtp
+	# Nexus 5k and 1000v do not support vtp
 	if (!/^VTP Operating Mode\s+:\s+(Transparent|Server)/) {
 	    $DO_SHOW_VLAN = 0;
 	}
@@ -904,7 +935,37 @@ REPEAT:	tr/\015//d;
 		ProcessHistory("SNMPSERVERCOMM","keysort","$_","$_") && next;
 	    }
 	}
-
+	# order/prune snmp-server host statements
+	# we only prune lines of the form
+	# snmp-server host a.b.c.d <community>
+	if (/^snmp-server host (\d+\.\d+\.\d+\.\d+) /) {
+	    if ($filter_commstr) {
+		my($ip) = $1;
+		my($line) = "snmp-server host $ip";
+		my(@tokens) = split(' ', $');
+		my($token);
+		while ($token = shift(@tokens)) {
+		    if ($token eq 'version') {
+			$line .= " " . join(' ', ($token, shift(@tokens)));
+			if ($token eq '3') {
+			    $line .= " " . join(' ', ($token, shift(@tokens)));
+			}
+		    } elsif ($token eq 'vrf') {
+			$line .= " " . join(' ', ($token, shift(@tokens)));
+		    } elsif ($token =~ /^(informs?|traps?|(no)?auth)$/) {
+			$line .= " " . $token;
+		    } else {
+			$line = "!$line " . join(' ', ("<removed>",
+						 join(' ',@tokens)));
+			last;
+		    }
+		}
+		ProcessHistory("SNMPSERVERHOST","ipsort","$ip","$line\n");
+	    } else {
+		ProcessHistory("SNMPSERVERHOST","ipsort","$1","$_");
+	    }
+	    next;
+	}
 	# Sort snmp user and delete passwords.
 	if (/^snmp-server user (\S+) (\S+) auth md5 (\S+) priv (\S+) localizedkey$/) {
 	    if ($filter_pwds >= 2) {
