@@ -1,9 +1,7 @@
 package panos;
 ##
-## $Id: panos.pm.in 3226 2016-01-15 21:41:54Z heas $
-##
-## rancid 3.6.1
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package panos;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -70,7 +62,7 @@ no warnings 'uninitialized';
 require(Exporter);
 our @ISA = qw(Exporter);
 
-use rancid 3.6.1;
+use rancid 3.12;
 
 @ISA = qw(Exporter rancid main);
 #XXX @Exporter::EXPORT = qw($VERSION @commandtable %commands @commands);
@@ -84,7 +76,7 @@ sub import {
 sub init {
 
     # add content lines and separators
-    ProcessHistory("","","","!RANCID-CONTENT-TYPE: $devtype\n!\n");
+    ProcessHistory("","","","#RANCID-CONTENT-TYPE: $devtype\n#\n");
 
     0;
 }
@@ -166,6 +158,23 @@ sub ShowInfo {
     return(0);
 }
 
+# This routine parses "show chassis inventory"
+sub ShowInventory {
+    my($INPUT, $OUTPUT, $cmd) = @_;
+    my($slot);
+
+    print STDERR "    In ShowInventory:: $_" if ($debug);
+
+    while (<$INPUT>) {
+	tr/\015//d;
+	last if (/^$prompt/);
+	return(1) if (/^Invalid syntax/);
+
+	ProcessHistory("INV","","","#$_");
+    }
+    ProcessHistory("INV","","","#\n");
+    return(0);
+}
 
 # This routine parses "show config running"
 sub ShowConfig {
@@ -176,9 +185,18 @@ sub ShowConfig {
 	tr/\015//d;
 	if (/^}\s*$|\[edit\]/) {
 	    $found_end = 1;
+	    ProcessHistory("","","","$_");
 	    return(1);
 	}
 
+	if (/^(\s*snmp-community-string )[^ ;]+(\s?[;{])$/ && $filter_commstr) {
+	    $_ = "#$1<removed>$2\n";
+	}
+	# filter protocol encryption keys
+	if (/^(\s*key )[^ ;]+(\s?[;{])$/ && $filter_pwds >= 1) {
+	    $_ = "#$1<removed>$2\n";
+	}
+
 	ProcessHistory("","","","$_");
 	# end of config
     }
