@@ -1,9 +1,7 @@
 package rancid;
 ##
-## $Id: rancid.pm.in 3520 2016-12-01 00:26:24Z heas $
-##
-## rancid 3.6.2
-## Copyright (c) 1997-2016 by Henry Kilmer and John Heasley
+## rancid 3.12
+## Copyright (c) 1997-2019 by Henry Kilmer and John Heasley
 ## All rights reserved.
 ##
 ## This code is derived from software contributed to and maintained by
@@ -18,15 +16,9 @@ package rancid;
 ## 2. Redistributions in binary form must reproduce the above copyright
 ##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-## 3. All advertising materials mentioning features or use of this software
-##    must display the following acknowledgement:
-##        This product includes software developed by Henry Kilmer and John
-##        Heasley and its contributors for RANCID.
-## 4. Neither the name of RANCID nor the names of its
+## 3. Neither the name of RANCID nor the names of its
 ##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
-## 5. It is requested that non-binding fixes and modifications be contributed
-##    back to RANCID.
 ##
 ## THIS SOFTWARE IS PROVIDED BY Henry Kilmer, John Heasley AND CONTRIBUTORS
 ## ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -61,6 +53,7 @@ use 5.010;
 use strict 'vars';
 use warnings;
 use 2.006 Socket qw(AF_INET AF_INET6 inet_pton);
+use File::Spec;
 require(Exporter);
 our @ISA = qw(Exporter);
 
@@ -84,6 +77,7 @@ our $host;				# hostname from cmdline
 our $clean_run;				# successful run
 our $found_end;				# found end of the config
 our $timeo;				# login script timeout in seconds
+our $timeodef;				# login script timeout in seconds dflt
 our $prompt;				# device prompt regex
 
 
@@ -96,14 +90,16 @@ our @EXPORT = qw($VERSION $devtype $script $lscript @modules $inloop
 		%commands @commands $commandcnt
 		$commandstr $cmds_regexp
 		$filter_commstr $filter_osc $filter_pwds $aclsort $aclfilterseq
-		$log $debug $file $host $clean_run $found_end $timeo $prompt
-		rancidinit loadtype
-		ProcessHistory numerically keynsort keysort valsort numsort
-		ipsort ipaddrval sortbyipaddr
-		RunCommand);
+		$log $debug $file $host $clean_run $found_end $timeo $timeodef
+		$prompt
+
+		bytes2human diskszsummary human2bytes
+		ipsort ipaddrval loadtype
+		keynsort keysort numerically numsort ProcessHistory rancidinit
+		RunCommand sortbyipaddr valsort which);
 
 BEGIN {
-    $VERSION = '3.6.2';
+    $VERSION = '3.12';
 }
 
 # initialize common rancid parameters and handle command-line arguments.
@@ -111,7 +107,7 @@ sub rancidinit {
     my($i);
 
     if ($main::opt_V) {
-	print "rancid 3.6.2\n";
+	print "rancid 3.12\n";
 	exit(0);
     }
 
@@ -123,12 +119,13 @@ sub rancidinit {
 
     $log = $main::opt_l;
     $debug = $main::opt_d;
-    $devtype = $main::opt_t;
+    $devtype = lc($main::opt_t);
     $file = $main::opt_f;
     $host = defined($ARGV[0]) ? $ARGV[0] : "";
     $clean_run = 0;
     $found_end = 0;
-    $timeo = 90;			# login script timeout in seconds
+    $timeodef = 90;			# login script timeout in seconds
+    $timeo = $timeodef;			# login script timeout in seconds
 
     $aclsort = "ipsort";		# ACL sorting mode
 
@@ -151,6 +148,8 @@ sub rancidinit {
     # determine oscillating data filtering mode
     if (defined($ENV{"FILTER_OSC"}) && $ENV{"FILTER_OSC"} =~ /no/i) {
 	$filter_osc = 0;
+    } elsif (defined($ENV{"FILTER_OSC"}) && $ENV{"FILTER_OSC"} =~ /all/i) {
+	$filter_osc = 2;
     } else {
 	$filter_osc = 1;
     }
@@ -216,10 +215,35 @@ sub loadtype {
 	    next if ($type ne $devtype);
 	    if (!$matched) {
 		$matched++;
-		printf(STDERR "loadtype: found device type %s in %s\n",
-		       $devtype, $file) if ($debug);
+		printf(STDERR "loadtype: found device type %s at %s:%d\n",
+		       $devtype, $file, $line) if ($debug);
 	    }
-	    if ($directive eq "script") {
+	    if ($directive eq "alias") {
+		# an alias maps a name to another, like a DNS CNAME; it does
+		# NOT augment
+		printf(STDERR
+		       "loadtype: found device type %s aliased to %s\n",
+		       $devtype, $value) if ($debug);
+		if (length($value) < 1) {
+		    printf(STDERR
+			   "loadtype: type %s alias has no value at %s:%d\n",
+			   $devtype, $file, $line);
+		    close(INPUT);
+		    return(-1);
+		}
+		$value =~ tr/[A-Z]/[a-z]/;
+		if ($devtype eq $value) {
+		    printf(STDERR
+			   "loadtype: type %s aliased to itself at %s:%d\n",
+			   $devtype, $file, $line);
+		    close(INPUT);
+		    return(-1);
+		}
+		close(INPUT);
+		undef($script); undef($lscript); undef(@modules);
+		undef($inloop); undef(@commandtable); $timeo = $timeodef;
+		return(loadtype($value));
+	    } elsif ($directive eq "script") {
 		$script = $value;
 	    } elsif ($directive eq "login") {
 		$lscript = $value;
@@ -229,6 +253,8 @@ sub loadtype {
 		$inloop = $value;
 	    } elsif ($directive eq "command") {
 		push(@commandtable, {$value2, $value});
+	    } elsif ($directive eq "timeout") {
+		$timeo = $value;
 	    } else {
 		printf(STDERR "loadtype: unknown directive %s at %s:%s\n",
 		       $directive, $file, $line);
@@ -287,11 +313,37 @@ sub loadtype {
     }
 
     if ($main::opt_C) {
+	my($cmd, @script);
+
+	if (!defined($script) || length($script) < 1) {
+	    printf(STDERR "$devtype: login script not defined\n");
+	    exit(1);
+	}
+
+	# if the first word of $script is not us (this script), exec the given
+	# script.
+	@script = split(/\s+/, $script);
+	if (which($script[0]) ne which($0)) {
+	    # -[hV] are not handled; they will have already been handled sooner.
+	    push(@script, "-d") if $debug;
+	    push(@script, "-l") if $log;
+	    push(@script, "-C") if $main::opt_C;
+	    push(@script, "-f") if $rancid::file;
+	    push(@script, $host);
+	    exec(join(" ", @script)) ||
+		printf(STDERR "exec($script[0]) failed: $!\n");
+	    exit(-1);
+	}
+
 	if (!defined($lscript)) {
 	    printf(STDERR "$devtype: login script not defined\n");
 	    exit(1);
 	}
-	print "$lscript -t $timeo -c \'$commandstr\' $host\n";
+	$cmd = $lscript;
+	if (defined($timeo)) {
+	    $cmd .= " -t $timeo";
+	}
+	print "$cmd -c \'$commandstr\' $host\n";
 	exit(0);
     }
 
@@ -328,6 +380,72 @@ sub ProcessHistory {
     1;
 }
 
+# convert bytes to human-readable format (ie GB/MB/KB/etc)
+sub bytes2human {
+    my($cnt) = @_;
+
+    if ($cnt >= (1024 * 1024 * 1024)) {
+	$cnt = int($cnt / (1024 * 1024 * 1024));
+	return("$cnt GB");
+    } elsif ($cnt >= (1024 * 1024)) {
+	$cnt = int($cnt / (1024 * 1024));
+	return("$cnt MB");
+    } elsif ($cnt >= (1024)) {
+	$cnt = int($cnt / 1024);
+	return("$cnt KB");
+    } elsif ($cnt > 0) {
+	return("<1KB");
+    } else {
+	return("0 KB");
+    }
+}
+
+# summarize total flash/disk space as human-readable form and remaining space
+# as a percentage.  unknown variables should be 'undef'.  returns a string,
+# without a trailing <CR>.
+sub diskszsummary {
+    my($total, $free, $used) = @_;
+    my($pcnt);
+
+    # remove commas
+    $total = human2bytes($total) if (defined($total));
+    $free = human2bytes($free) if (defined($free));
+    $used = human2bytes($used) if (defined($used));
+
+    if (defined($free)) {
+	$pcnt = int($free / $total * 100);
+    } elsif (defined($used)) {
+	$pcnt = int(($total - $used) / $total * 100);
+    } else {
+	$total = $free + $used;
+	$pcnt = int($free / $total * 100);
+    }
+
+    $total = bytes2human($total);
+
+    return("$total total ($pcnt% free)");
+}
+
+# convert human-readable format (like "300,024 GB/MB/KB/etc") to bytes
+sub human2bytes {
+    my($cnt) = @_;
+    my($mp);
+
+    $cnt =~ s/,//g;
+
+    if ($cnt =~ /gb/i) {
+	$mp = 1024 * 1024 * 1024;
+    } elsif ($cnt =~ /mb/i) {
+	$mp = 1024 * 1024;
+    } elsif ($cnt =~ /kb/i) {
+	$mp = 1024;
+    } else {
+	$mp = 1;
+    }
+    $cnt =~ /(\d+)/;
+    return($1 * $mp);
+}
+
 sub numerically { $a <=> $b; }
 
 # keynsort(%hash) is a sort routine that will sort numerically on the
@@ -427,8 +545,27 @@ sub sortbyipaddr {
     &ipaddrval($a) cmp &ipaddrval($b);
 }
 
-# This routine parses a single command that returns no required info.  The
-# output is discarded.
+# if $prog is non-FQPN, look for it in PATH and return the ABS-FQPN, else
+# just return the ABS-FQPN.
+sub which {
+    my($prog) = @_;
+    my($path);
+
+    # ABS or relative PATH (begins with /, ./, or ../)
+    if ($prog =~ /^(\/|\.\/|\.\.\/)/) {
+	return(File::Spec->rel2abs($prog));
+    }
+    for $path (split(/:/, $ENV{"PATH"})) {
+	$path = join('/', ($path, $prog));
+	if (-f $path) {
+	    return(File::Spec->rel2abs($path));
+	}
+    }
+    return($prog);
+}
+
+# This routine parses a single command that returns no required info; the
+# input is discarded.
 sub RunCommand {
     my($INPUT, $OUTPUT, $cmd) = @_;
     print STDERR "    In RunCommand: $_" if ($debug);
